title: Going "Write-Only"
url: http://begriffs.com/posts/2015-04-20-going-write-only.html
hash_url: 73a06d869ab721933adaaa7203dcaa62

<p><a href="http://joey.hess.usesthis.com/">Joey Hess</a> inspires me. He lives in a cabin and programs Haskell on a drastically under-powered netbook. He harvests all his electricity from the sun with a homemade electrical system. Joey has designed his computing life to be highly distributed like the git version control he loves. His internet connection is intermittent and slow, his screen resolution incredibly small, but I think these seeming handicaps are actually the key to his maintaining perspective and accomplishing the mission – in his own words – of “building worthwhile things that might last.”</p>
<p>Think of what that really means. Things that last. It’s all the more surprising because Joey is talking about writing software. Most code is notoriously ephemeral. It is complicated and ties in tightly with its environment. When the environment changes even a little the code becomes unusable without maintenance. By contrast the written word, math, music, or visual art are shared between human minds (much more forgiving systems, minds) and don’t tarnish as easily as code.</p>
<p>The programmer known as “_why the lucky stiff” shares this concern:</p>
<blockquote>
<p>To program any more was pointless. My programs would never live as long as [Kafka’s] The Trial. A computer will never live as long as The Trial. … What if Amerika was only written for 32-bit Power PC? Can an unfinished program be reconstructed? Can I write a program and go, “Ah, well, you get the gist of it.” … But no. It wasn’t written for 32-bit Power PC. It was written for eyes.</p>
</blockquote>
<p>These people’s thoughts are not idle for me. They contain a reproach, a warning that one can be very busy and yet do unproductive things, hamartia. I want to focus on doing the right thing. Actually focus is the wrong word. Focusing my thoughts would imply the same thoughts but sharper, whereas I want to change the way I think.</p>
<p>Thoreau described this mental recaliberation in <a href="http://thoreau.eserver.org/life2.html">Life Without Principle</a>:</p>
<blockquote>
<p>If we have thus desecrated ourselves, — as who has not? — the remedy will be by wariness and devotion to reconsecrate ourselves, and make once more a fane of the mind. We should treat our minds, that is, ourselves, as innocent and ingenuous children, whose guardians we are, and be careful what objects and what subjects we thrust on their attention. Read not the Times. Read the Eternities.</p>
</blockquote>
<p>Whereas the above writers were all reclusive, I enjoy being around people and would like to blend a city environment with a consecration of the mind. Can I be resolute yet approachable? Perhaps yes, and perhaps it is a greater feat than solitary authenticity. Emerson <a href="https://www.gutenberg.org/files/16643/16643-h/16643-h.htm#SELF-RELIANCE">remarked</a> that</p>
<blockquote>
<p>It is easy in the world to live after the world’s opinion; it is easy in solitude to live after our own; but the great man is he who in the midst of the crowd keeps with perfect sweetness the independence of solitude.</p>
</blockquote>
<p>My experiment, then, is to arrange my life this month (and possibly beyond) to avoid contact with cheap ephemeral things and become steeped in quality thought and art. How do I know what to ignore, you ask? When in doubt I’ll leverage the survivorship bias: most old things that people still talk about are likely to be quality; the dumb old things are forgotten.</p>
<p>The simple test of time does yield false negatives. People are doing amazing work every day. But creativity needn’t be apprised of every leaf that falls. Donald Knuth said it best,</p>
<blockquote>
<p>Email is a wonderful thing for people whose role in life is to be on top of things. But not for me; my role is to be on the bottom of things. What I do takes long hours of studying and uninterruptible concentration. I try to learn certain areas of computer science exhaustively; then I try to digest that knowledge into a form that is accessible to people who don’t have time for such study.</p>
</blockquote>
<p>My first concrete step will be to eliminate variable information rewards from my computing life. These are repeated activities which occasionally – and unpredictably – give a pleasant surprise. A nontechnical example is playing slot machines. In my case it would be checking email or social media. Email is a wonderful thing but its unregulated variable rewards create compulsion. For the experiment I will check email once per week, on Monday. I’ll enable an email auto-responder for the other days to explain the setup so that nobody thinks I’m ignoring them in particular.</p>
<p>On Twitter and Github I’ll be entirely write-only. I’ll check replies/messages/issues on Mondays along with my email. For Twitter I’ll use the <a href="https://github.com/sferik/t">t</a> command line client to make posting write-only updates easy and non-distracting.</p>
<p>I will eliminate all use of the computer that is not directly related to creating things. If I’m not coding, writing, or editing videos then there will be literally nothing to do. I am going to dissociate the computer from mindless fun, from the capacity to kill time online. Without the (false?) feeling of connectedness through a glowing rectangle think how quiet and dull it can be sitting inside. Ultimately I’ll be forced by boredom to read or go outside.</p>
<p>Who can justify soaking up random online news at the expense of a neverending tidal wave of fine works from the the ocean of history? I will keep outstanding books on hand at all times, books from reading lists like <a href="http://www.sjc.edu/academic-programs/undergraduate/seminar/annapolis-undergraduate-readings/">St John’s College</a> and <a href="http://www.shimer.edu/live/files/105-academic-catalog-20132015--final-61913pdf">Shimer</a>. This way I can sublimate my urge to browse the web by instead making progress in some of the fullest and best expressions of ideas.</p>
<p>The self-censorship goes far beyond choosing classic books over web browsing. When I code I often listen to music, and music influences attitude. Choice of music affects the mind. Plato said in the Republic that,</p>
<blockquote>
<p>…rhythm and harmony most of all insinuate themselves into the inmost part of the soul and lay hold of it in bringing grace with them; and they make a man graceful if he is correctly reared, if not, the opposite.</p>
</blockquote>
<p>The discussion in The Republic maintains that would-be citizens of the ideal republic should be exposed to music that cultivates their good qualities, and prohibited from listening to the bad. Much modern music creates agitation and aggression. I’ll listen to serene and balanced songs like Gregorian chant and neoclassical, preferably from my own recordings rather than online streaming.</p>
<p>To further insulate myself from the pernicious influence of online mediocrity I will disable image loading in my browser. Online pictures are of two kinds: mundane photographs and a simplified telegraphic advertising style, for logos and minimal ornament. The “product style” is made of highly saturated homogeneous clean shapes and serves merely as simple mnemonics for products or services. The craftsmanship is these images is intentionally low because the images are meant to recall a product as efficiently as possible.</p>
<p>Extended time on the internet inures us to constraint and simplification. Take your average startup company logo and compare it with the free expression of a skilled painter. The logo looks cheap and comical, a petty token designed to evoke unthinking trust and visual association. It is easier than ever before in history to publish information worldwide, but paradoxically we produce lower quality work. Or maybe it’s the survivorship bias again. Think of a nineteenth century book with ornate etched print illustrations. The etching, and even the pressing and collating of pages were difficult processes but somehow the artists outdid us, we who can so easily create, modify and distribute images. Goodbye cartoonish web images, let me be immersed in nature and see uninhibited art instead.</p>
<p>These little rules constitute a kind of “morality,” as Nietzsche described it, a disciplinary means to attain strength.</p>
<blockquote>
<p>There should be long obedience in the same direction, there thereby results, and has always resulted in the long run, something which has made life worth living.</p>
</blockquote>
<p>I choose living among quality old things for the sake of creation, hoping by their example to attain quality of my own. Check back on this blog to see how it goes for me – unless of course I’ve inspired you to go write-only too.</p>
